package org.jenkinsci.plugins.p4_client.workspace;

import hudson.Extension;
import hudson.model.AutoCompletionCandidates;
import hudson.util.FormValidation;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

import org.jenkinsci.plugins.p4_client.client.CharsetTypes;
import org.jenkinsci.plugins.p4_client.client.ConnectionFactory;
import org.kohsuke.stapler.DataBoundConstructor;
import org.kohsuke.stapler.QueryParameter;

import com.perforce.p4java.core.IStream;
import com.perforce.p4java.core.IStreamSummary;
import com.perforce.p4java.option.server.GetStreamsOptions;
import com.perforce.p4java.server.IOptionsServer;

public class StreamWorkspaceImpl extends Workspace {

	private final String charset;
	private final String streamName;
	private final String prefix;

	public String getCharset() {
		return charset;
	}

	public String getStreamName() {
		return streamName;
	}

	public String getPrefix() {
		return prefix;
	}

	@Override
	public WorkspaceType getType() {
		return WorkspaceType.STREAM;
	}

	@DataBoundConstructor
	public StreamWorkspaceImpl(String charset, String streamName, String prefix) {
		this.charset = charset;
		this.streamName = streamName;
		this.prefix = prefix;
	}

	@Extension
	public static final class DescriptorImpl extends WorkspaceDescriptor {

		@Override
		public String getDisplayName() {
			return "Streams (view generated by Perforce for each node)";
		}

		/**
		 * Provides auto-completion for workspace names. Stapler finds this
		 * method via the naming convention.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteStreamName(
				@QueryParameter String value) {

			AutoCompletionCandidates c = new AutoCompletionCandidates();
			try {
				IOptionsServer iserver = ConnectionFactory.getConnection();
				if (iserver != null && value.length() > 1) {
					List<String> streamPaths = new ArrayList<String>();
					streamPaths.add(value + "...");
					GetStreamsOptions opts = new GetStreamsOptions();
					opts.setMaxResults(10);
					List<IStreamSummary> list = iserver.getStreams(streamPaths,
							opts);
					for (IStreamSummary l : list) {
						c.add(l.getStream());
					}
				}
			} catch (Exception e) {
			}

			return c;
		}

		public FormValidation doCheckStreamName(@QueryParameter String value) {
			try {
				IOptionsServer p4 = ConnectionFactory.getConnection();
				IStream stream = p4.getStream(value);
				if (stream != null) {
					return FormValidation.ok();
				}
				return FormValidation.error("Unknown Stream: " + value);
			} catch (Exception e) {
				return FormValidation.error("Perforce connection error.");
			}
		}

		public FormValidation doCheckPrefix(@QueryParameter final String value) {
			if (value == null)
				return FormValidation.error("Prefix path is mandatory.");

			Pattern regex = Pattern.compile(".*[-_\\.]$");
			if (regex.matcher(value).matches())
				return FormValidation.ok();

			return FormValidation.error("Prefix must end with '-', '_' or '.'");
		}
		
		/**
		 * Magic Stapler method; provides auto-completion for Charset types.
		 * 
		 * @param value
		 *            The text that the user entered.
		 */
		public AutoCompletionCandidates doAutoCompleteCharset(
				@QueryParameter String value) {
			AutoCompletionCandidates c = new AutoCompletionCandidates();
			for (String set : CharsetTypes.getCharsetsSince(20133)) {
				if (set.startsWith(value)) {
					c.add(set);
				}
			}
			return c;
		}
	}

	/**
	 * Builds Workspace name from prefix, node and project.
	 * 
	 * @return
	 */
	@Override
	public String getName() {
		StringBuffer sb = new StringBuffer();
		sb.append(prefix);
		sb.append((getNodeName() != "") ? getNodeName() : "master");
		sb.append("-");
		sb.append(getProjectName());
		return sb.toString();
	}
}